/***************************************************************************
                          filespanel.h  -  description
                             -------------------
    begin                : sun oct 27 2002
    copyright            : (C) 2002 by Piotr Mierzwiski
    email                : peterm@go2.pl

    copyright            : See COPYING file that comes with this project
 ***************************************************************************/


#ifndef _FILESPANEL_H_
#define _FILESPANEL_H_

#include "keyshortcuts.h"
#include "listviewext.h"
#include "devicesmenu.h"
#include "urlinfoext.h"
#include "virtualfs.h"
#include "statusbar.h"
#include "pathview.h"
#include "enums.h"

#include "plugins/fp/settings/listviewsettings.h"
#include "plugins/fp/settings/otherappsettings.h"
#include "plugins/fp/settings/filesystemsettings.h"

#include <qtimer.h>

/**
 * Author:
 * Piotr Mierzwiski
 **/
/**
 @short Klasa odpowiedzialna za obsug panela plikw.
 Tworzone s tutaj obiekty skadowe panela, tj. widok cieki, widok listy
 plikw oraz pasek statusu. Inicjowany jest tutaj wygld panela. Zawiera
 obsug klawiatury. Wystpuj tutaj funkcje pokazujce rne dodatkowe
 informacje, tj. list ulubionych, list napdw, statystyk dla biecego
 dysku. Poza tym ma sloty, ktre powoduj zastosowanie ustawie dla listy,
 systemw plikw i innych ustawie aplikacji. Mieci si tutaj take slot
 uruchamiany po kadym zakoczeniu operacji (rekurencyjnej i zwykej), jego
 gwnym zadaniem jest uaktualnianie widoku listy.
 */
class FilesPanel : public QWidget
{
	Q_OBJECT
public:
	/** Konstruktor klasy.\n
	 @param initURL - cieka lub nazwa pliku, ktry ma by zaadowany do panelu,
	 @param parent - wskanik na rodzica dla obiektu,
	 @param name - nazwa dla obiektu.\n
	 Inicjowane s tu niezbdne skadowe klasy, tworzone obiekty: widoku listy,
	 widoku cieki oraz paska statusu, a take inicjowany (asynchronicznie) widok
	 panela.
	 */
	FilesPanel( const QString & initURL, QWidget *parent, const char *name );

	/** Destruktor klasy.\n
	 Wywoywane s tutaj funkcje powodujce zapisanie ustawie listy i panela do
	 pliku konfiguracyjnego oraz usunicie wszystkich utworzonych wczeniej
	 obiektw.
	 */
	~FilesPanel();

	/** Zwraca informacj o rodzaju systemu plikw w panelu.\n
	 @return rodzaj systemu plikw, patrz: @see KindOfFilesSystem.
	 */
	KindOfFilesSystem kindOfCurrentFS() const { return mVirtualFS ? mVirtualFS->kindOfCurrentFS() : UNKNOWNfs; }

	/** Zwraca rodzaj widoku w panelu.
	 @return rodzaj widoku, patrz: @see KindOfListView.
	*/
	int kindOfView() const { return mKindOfListView; }

	/** Zwraca informacj o tym czy panel posiada focus.\n
	 @return TRUE oznacza posiadanie focus-a, FALSE - brak.
	 */
	bool hasFocus() const { return mVirtualFS ? mVirtualFS->hasFocus() : FALSE; }

	/** Ustawia focus dla panela (wawicie dla widoku listy).
	 */
	void setFocus() { if ( mListViewExt )  mListViewExt->setFocus(); }


	/** Zwraca informacj o ciece do otwartego w panelu katalogu.\n
	 @return bieca cieka.
	 */
	QString currentURL() const { return mVirtualFS ? mVirtualFS->currentURL() : QString("");  }

	/** Metoda powoduje zmian rozmiaru podanej kolumny widoku listy plikw.\n
	 @param section - numer kolumny, ktrej jest zmieniany rozmiar,
	 @param oldSize - poprzedni rozmiar biecej kolumny,
	 @param newSize - nowy rozmiar biecej kolumny.\n
	 Fukncja wykorzystywana przy synchronizacji kolumn w obu widokach list.
	 */
	void resizeSection( uint section, uint oldSize, uint newSize ) { mListViewExt->slotHeaderSizeChange( section, oldSize, newSize ); }

	/** Metoda ustawia skadow zawierajc ciek do panela obok.\n
	 @param url - nowa cieka.
	 */
	void setPanelBesideURL( const QString & url ) { mPanelBesideURL = url; }

	/** Metoda przywracajca geometri przy ponownym pokazaniu widoku panela.\n
	 Konieczne jest jej wywoanie po zmianie widoku z osadzonego podgldu pliku
	 na widok panela.
	 */
	void updateView();

	/** Funkcja uruchamia procedur kopiowania lub przenoszenia plikw.\n
	 @param filesList - lista plikw do kopiowania bd przenoszenia,
	 @param move - ustawiona na TRUE wymusza przenoszenie zamiast kopiowania.\n
	 Wywoywana jest tutaj tylko metoda @em mVirtualFS::copyFilesTo(), ktrej
	 parametrami s: biecy URL, lista plikw oraz status przenoszenia.\n \n
	 */
	void copyFiles( const QStringList & filesList, bool move );

	/** Metoda powoduje uaktualnienie listy.\n
	 @param updateListOp - rodzaj uaktualnienia, patrz: @see UpdateListOperation,
	 @param list - wskanik na list elementw, ktrymi naley uaktualni biec,
	 @param newUrlInfo - informacja o pliku (uywana jeli uaktualniany jest jeden
	 element).\n
	 Jest ona uywana do uakatualniania informacji w panelu obok.
	 */
	void updateList( int updateListOp, ListViewExt::SelectedItemsList * selectedItemsList, const UrlInfoExt & newUrlInfo ) {
		mListViewExt->updateList( updateListOp, selectedItemsList, newUrlInfo );
	}

	/** Zwraca wskanik na list zaznaczonych elementw.\n
	 @return wskanik do listy zaznaczonych typu @em SelectedItemsList
	 */
	ListViewExt::SelectedItemsList * selectedItemsList() {
		return mListViewExt->selectedItemsList();
	}

	/** Ustawia wskanik do obiektu skojae plikw.\n
	 @param fa - wskanik do obiektu skojae plikw.
	 */
	void setFilesAssociation( FilesAssociation *fa ) {
		if ( mVirtualFS ) mVirtualFS->setFilesAssociation( (mFilesAssociation=fa) );
	}

	/** Ustawia wskanik do obiektu skrtw klawiszowych panela plikw.\n
	 @param keyShortcuts - wskanika na obiekt skrtw klawiszowych.
	 */
	void setKeyShortcuts( KeyShortcuts * keyShortcuts ) {
		mKeyShortcuts = keyShortcuts;
	}

	/** Metoda uaktualnia menu zawierajce list filtrw.\n
	 @param filters - wskanik na obiekt filtrw.
	 */
	void updateFiltersMenu( Filters * filters );

	/** Otwiera biecy plik (element na ktrym stoi kursor) w podanym trybie.\n
	 @param editMode - tryb otwarcia pliku.
	 */
	void openCurrentFile( bool editMode ) {
		if ( mVirtualFS && mListViewExt ) mVirtualFS->openFile( mListViewExt->pathForCurrentItem(FALSE), editMode );
	}

	void createNewEmptyFile( bool createDir ) {
		if ( mVirtualFS ) mVirtualFS->createNewEmptyFile( createDir );
	}

private:
	PathView    *mPathView;
	StatusBar   *mStatusBar;
	ListViewExt *mListViewExt;
	FilesAssociation *mFilesAssociation;

	VirtualFS    *mVirtualFS;
	DevicesMenu  *mDevicesMenu;
	KeyShortcuts *mKeyShortcuts;

	QToolButton *mFilterButton;
	QPopupMenu  *mFiltersMenu;
	Filters *mFilters;

	uint mWidth;
	int  mKindOfListView;
	bool mShiftButtonPressed;
	QListViewItem *mCurrentItem;

	QString mInitUrl;
	QString mPanelBesideURL;
	QString mFinalPathOfTree;

	int  mFilterId;
	bool mFilterForFiltered;

	/** Metoda usuwa obiekt wirtualnego systemu plikw ( @em mVirtualFS ).
	*/
	void removeFS();

	/** Funkcja inicjuje ustawienia panela.\n
	 Na pocztku inicjowana jest, wartoci z pliku konfiguracyjnego, skadowa
	 okrelajca filtr dla listy plikw w panelu. Ustawiany jest rodzaj sortowania
	 listy (w klasie listy) - domylnie po nazwie rosnco. Okrelane jest (w klasie
	 listy) czy focus ma by stale widoczny (domylne ustawienie).
	 */
	void initPanel();

	/** Inicjowany jest tutaj (wartociami z pliku konfiguracyjnego) wygld listy.\n
	 Wczytywane s nastpujce ustawienia: informacje o kolumnach (ktre bd
	 pokazywane i jaki rozmiar ma kada), font dla listy, informacja o pokazywaniu
	 ikon na licie, o nie chowajcym si kursorze listy, o formacie rozmiarw
	 plikw, nazwa schematu kolorw i wartoci ich. W dalszej kolejnoci
	 nastpuje ustawienia tych wszystkich wasnoci na licie.
	 */
	void initLookOfList();

	/** Inicjowana jest tutaj lista dla menu guzika z filtrami listy plikw.
	 */
	void initFiltersButton();

	/** W metodzie tworzony jest nowy obiekt systemu plikw.\n
	 @param url - cieka do katalogu.\n
	 Na pocztku usuwana jest stara lista, po czym ustawiany (dla klasy listy)
	 filtr plikw oraz rodzaj listy (zwyka/drzewo). Nastpnie sprawdzany jest
	 docelowy rodzaj systemu plikw, i podstawie tej informacji tworzony obiekt
	 klasy odpowiedniego systemu plikw (LOCALfs, ARCHIVEfs, FTPfs, itp.).
	 Po bezproblemowym utworzeniu tego obiektu ustanawiane s poczenia sygnaw
	 ze slotami i wywoywana funkcja uaktualniajca statusu panela.
	 */
	void createFS( const QString & url );

protected:
	/** Zapisywane s tutaj niektre ustawienia panela.\n
	 Mianowicie informacje o: kolumnach, nieukrywaniu kursora podczas przesuwania
	 widoku listy, pokazywaniu ikon oraz rodzaju widoku.
	 */
	void saveSettings();

	/** Obsuga zdarzenia zmiany rozmiaru panela.\n
	 Dostosowywane s tutaj rozmiary elementw biecego widgetu, tj. widoku
	 cieek, panela z list/drzewem oraz widgetu z info o licie ('mStatusBar').
	 Uruchamiana jest te procedura uaktualniajca status listy plikw.
	 */
	void resizeEvent( QResizeEvent * );

	/** Metoda zawiera obsug klawiatury dla panelu.\n
	 */
	void keyPressEvent( QKeyEvent *k );

public slots:
	/** Slot powoduje zmian biecego widoku na podany.\n
	 @param kindOfListView - rodzaj widoku, patrz: @see KindOfListView.
	*/
	void slotChangeKindOfView( uint kindOfListView );

	/** Slot powoduje pokazanie dialogu menadera ulubionych.
	 */
	void slotShowFavoriteManager();

	/** Slot powoduje pokazanie dialogu menadera filtrw dla plikw.
	 */
	void slotShowFiltersManager();

	/** Powoduje dodanie biecego katalogu do ulubionych.
	 */
	void slotAddCurrentToFavorites();

	/** Slot powoduje pokazanie, menu z prost statystyk dotyczc biecego
	 dysku.\n Wystpuj tam nastpujce wartoci:
	  @li cakowita iloc wolnych bajtw
	  @li cakowita ilo dostpnych bajtw.\n
	 Przy czym wartoci te s zaokrglane do liczby, ktra ma wag (KB/MB/GB) i
	 dwch miejsc po przecinku. Ostatnia opcja pojazuje statystyk dla wszystkich
	 dostpnych dyskw. Menu pokazywane jest w lewym grnym rogu panelu.
	 */
	void slotShowSimpleDiscStat();

	/** Slot wykonuje tzw. szybkie wyjcie z systemu plikw rnego ni lokalny.\n
	 Odczytywana jest w tym celu poprzednia (w histori) cieka z lokalnego
	 systemu plikw.
	 */
	void slotQuickGetOutFromFS();

	/** Slot powoduje pokazanie dialogu menadera pocze FTP.\n
	 Po wciniciu "Connect", wykonywana jest prba poczenia.
	 */
	void slotShowFtpManagerDlg();

	/** Slot powoduje ponowne odczytanie biecego katalogu.
	 */
	void slotRereadCurrentDir();

	/** Slot powoduje pokazanie menu z list dostpnych urzdze pamici masowej
	 (tj. napdy CDROM, HDD, itp.).\n
	 Menu jest pokazywane w lewym grnym rogu panelu.
	 */
	void slotShowDevicesMenu();

	/** Slot powoduje pokazanie menu zawierajce list ulubionych katalogw i opcj
	 do jej konfiguracji.\n
	 Menu jest pokazywane w lewym grnym rogu biecego panelu.
	 */
	void slotShowFavorites();

	/** Slot powoduje wywoanie dialogu wyszukiwania pliku.
	 */
	void slotFindFile();


	/** Slot powoduje zmian biecego katalogu na podany.\n
	 @param inputURL - nowa cieka do katalogu.\n
	 Przy czym, jeli cieka wskazuje na inny ni biecy system plikw, wtedy
	 nastpuje jego utworzenie. Slot obsuguje sygna z klasy PathView.
	*/
	void slotPathChanged( const QString & inputURL );


	/** Slot uruchamia procedur usuwania zaznaczonych plikw.\n
	 Wywoywana jest tutaj tylko metoda: @see VirtualFS::removeFiles().
	 */
	void slotDelete();

	/** Slot uruchamia procedure tworzenia linku dla biecego pliku.\n
	 Wywoywana jest tutaj tylko metoda: @em VirtualFS::makeLink() z parametrem,
	 ktry jest ciek do katalogu w ssiednim panelu.
	 */
	void slotMakeLink();

	/** Slot uruchamia procedure edycji linku dla biecego pliku.\n
	 Wywoywana jest tutaj tylko metoda: @em VirtualFS::makeLink() z parametrem,
	 ktry jest ciek do katalogu w ssiednim panelu.
	 */
	void slotEditLink();

	/** Slot powoduje pokazanie dialogu waciwoci dla wybranych plikw.\n
	 W tym celu wywoywana jest metoda: @em mVirtualFS::showPropertiesDialog().
	 */
	void slotShowProperties();

	/** Slot uruchamia procedur otwarcia biecego pliku lub katalogu.\n
	 Wywoywany jest w momencie wcinicia klawiszy Enter/Return, podwjnym
	 klikniciu na biecy element listy lub klikniciu w plusik wywietlany
	 na widoku drzewa przy nazwie katalogu. Uruchamia procedur otwarcia tego
	 elementu listy. Absolutn ciek inicjowana jest skadowa
	 @em mFinalPathOfTree oraz wywoywana @em metoda mListViewExt::open(), ktrej
	 parametrem jest powysza cieka.
	 */
	void slotReturnPressed( QListViewItem * );

	/** Funkcja inicjuje i ewentualnie uruchamia operacje kopiowania.\n
	 @param shiftPressed - rwny TRUE oznacza, ze przy wywoaniu slotu trzymany
	 by klawisz Shift. Wymusza on kopiowanie do tego samego katalogu,
	 @param move - ustawiony na TRUE wymusza przenoszenie zamiast kopiowania.\n
	 Uruchamiana jest tutaj procedura kopiowania do biecego bd ssiedniego
	 panela (jest to zalene od systemu plikw w biecym panelu).
	 */
	void slotCopy( bool shiftPressed, bool move );

	/** Slot powoduje otwarcie biecego pliku lub katalogu w ssiednim panelu.\n
	 @param dirForce - rwny TRUE, wymusza otwarcie katalogu, w przeciwnym razie
	 nastpuje prba otwarcia pliku.
	 */
	void slotOpenIntoPanelBeside( bool dirForce );


	/** Slot uruchamia procedur otwarcia archiwum.\n
	 @param fileName - nazwa archiwum do otwarcia.\n
	 Przed otwraciem sprawdzane jest czy podany plik to archiwum.
	 */
	void slotOpenArchive( const QString & fileName );


	/** Slot wykonuje zastosowanie ustawie dla widoku listy.\n
	 @param lvs - adres na obiekt z ustawieniami widoku listy.
	 */
	void slotApplyListViewSettings( const ListViewSettings & );

	/** Slot wykonuje zastosowanie ustawie dla tzw. pozostaych ustawie.\n
	 @param oas - adres na obiekt z tzw. pozostaymi ustawieniami.
	 */
	void slotApplyOtherAppSettings( const OtherAppSettings & oas );

	/** Slot wykonuje zastosowanie ustawie dla systemu plikw.\n
	 @param fss - adres na obiekt z ustawieniami systemu plikw.
	 */
	void slotApplyFileSystemSettings( const FileSystemSettings & fss );

	/** Slot uruchamia procedur wczytywania pliku do widoku.\n
	 @param fileName - nazwa pliku do wczytania,
	 @param buffer - adres bufora, w ktrym naley umieszcza wczytywane dane,
	 @param bytesRead - ilo wczytywanych na raz bajtw, a po odczycie zawiera
	 ilo faktycznie wczytanych bajtw (0 oznacza koniec pliku)
	 @param fileLoadMethod - metoda wczytywania pliku, patrz @see View::UpdateMode.\n
	 Uywany do wczytywania pliku otwartego w panelu obok.
	 */
	void slotReadFileToView( const QString & fileName, QByteArray & buffer, int & readBlockSize, int fileLoadMethod ) {
		if ( mVirtualFS ) mVirtualFS->slotReadFileToView( fileName, buffer, readBlockSize, fileLoadMethod );
	} // need for load file from panel beside

private slots:
	/** Slot suy do inicjowania widoku listy.\n
	 Jeli widokiem ma by FTP, wtedy pokazywany jest dialog poczenia, a po jego
	 zatwierdzeniu nastpuja prba poczenia. Jeli widokniem nie bdzie FTP, wtedy
	 otwierany jest katalog, ktrego nazwa zawarta jest w skadowej @em mInitUrl.
	 Slot wywoywany asynchronicznie z konstruktora.
	 */
	void slotInitPanelList();

	/** Slot uruchamia procedure tzw. szybkiej zmiany atrybutw dla biecego
	 pliku.\n
	 @param column - kolumna w ktrej naley zmieni warto atrybutu,
	 @param oldText - aktualny tekst z podanej kolumny,
	 @param newText - nowy tekst dla podanej kolumny.\n
	 Wywoywana jest tutaj metoda @em mVirtualFS::changeAttribut(), parametrami
	 podanymi do tego slotu.
	 */
	void slotRenameCurrentItem( int column, const QString & oldText, const QString & newText );

	/** Slot obsugujcy zakoczenie operacji na plikach.\n
	 @param operation - zakoczona operacja,
	 @param noError - status nie wystpienia bdu w zakoczonej operacji.\n
	 Uruchamiane s czynnoci, ktre naley wykona po zakoczeniu,
	 operacji, jest to min. uaktualnianie listy w obydwu panelach.
	 */
	void slotResultOperation( Operation, bool );

	/** Slot inicjuj parametr informacj o tym czy w biecym katalog aktualnie
	 zalogowany uytkownik moe zapisywa.\n
	 @param dw - zmienna, w ktrej zostanie zapisana informacja.\n
	 Ze slotu tego korzysta obiekt klasa PathView w celu wybrania koloru ta dla
	 siebie.
	 */
	void slotPossibleDirWritable( DirWritable & );

	/** Slot powoduje ustawienie podanej cieki w widoku cieki.\n
	 @param path - cieka do ustawienia.\n
	 Inicjowana jest rwnie skadowa klasy VirtualFS, okrelajca biec ciek.
	 Slot uywany jest przez klas ListViewExt, kiedy tam przy nawigacji po widoku
	 drzewa konieczna jest zmiana biecej cieki.
	 */
	void slotPathChangedForTheViewPath( const QString & path );

	/** Slot powoduje zastosowanie filtru o podany id dla listy plikw.\n
	 @param filterId - id filtra, ktry naley ustawi.
	 */
	void slotChangeFilter( int filterId );

signals:
	/** Sygna wywoywany w momencie zmiany rozmiaru kolumny widoku listy.\n
	 @param section - numer kolumny, ktrej rozmiar jest zmieniany,
	 @param oldSize - poprzedni rozmiar kolumny,
	 @param newSize - nowy rozmiar kolumny.
	 */
	void signalHeaderSizeChange( int section, int oldSize, int newSize );

	/** Sygna po zakoczeniu odczytywania listy plikw z nowego katalogu.\n
	 @param url - nowa cieka dla panela.\n
	 Powoduje on uaktualnienie informacji w panelu obok o biecej ciece.
	 */
	void signalSetPanelBesideURL( const QString & url );

	/** Sygna uruchamia procedur otwarcia pliku lub katalogu w panlu obok.\n
	 @param fileName - nazwa pliku lub katalogu do otwarcia.
	 */
	void signalOpenIntoPanelBeside( const QString & fileName );

	/** Sygna powoduje pobranie informacji o systemie plikw w panelu obok.\n
	 @param kofs - zmienna, ktra zostanie zainicjowana rodzajem systemu plikw.
	 */
	void signalGetKindOfFSInPanelBeside( KindOfFilesSystem & kofs );

	/** Sygna uruchamia procedur kopiowania lub przenoszenia inicjowan
	 z drugiego panela.\n
	 @param filesList - lista plikw do skopiowania/przeniesienia,
	 @param move - rwne TRUE wymusza operacj przenoszenia, w przeciwnym razie
	 wykonane zostanie kopiownie.\n
	 Operacja taka jest wymagana dla systemw plikw rnych od lokalnego.
	 */
	void signalCopyFiles( const QStringList & filesList, bool move );

	/** Sygna powoduje uaktualnienie listy w panelu obok.\n
	 @param updateListOp - rodzaj operacji uaktualniania, patrz:
	  @see ListViewExt::UpdateListOperation,
	 @param itemsList - wskanik do wskanika na list z elementami do
	 uaktualnienia,
	 @param newUrlInfo - informacje o pliku (uywane jeli jest jeden pliku
	 uaktualniany).
	 */
	void signalUpdateSecondPanel( int updateListOp, QValueList<QListViewItem *> * selectedItemsList, const UrlInfoExt & );

	/** Sygna powoduje zapisanie na podany wskanik do listy, wskanika do listy
	 zaznaczonych w panelu obok.\n
	 @param selectedItemsList - wskanik do adresu listy.\n
	 Jest on wykorzystywany przez systemy plikw rzne od lokalnego.
	 */
	void signalGetSILfromPanelBeside( QValueList<QListViewItem *> *& selectedItemsList );

	/** Sygna powoduje zastosowanie zmienionych skrtw klawiszowych w oknie
	 gwnym aplikacji.\n
	 @param keyShortcuts - wskanik na obiekt skrtw klawiszowych.
	 */
	void signalApplyKeyShortcut( KeyShortcuts * keyShortcuts );

};

#endif
