/***************************************************************************
                          listviewwidgets.h  -  description
                             -------------------
    begin                : fri mar 26 2004
    copyright            : (C) 2003 by Piotr Mierzwiski
    email                : peterm@go2.pl

    Copyright: See COPYING file that comes with this project
***************************************************************************/

#ifndef LISTVIEWWIDGETS_H
#define LISTVIEWWIDGETS_H

#include <qlistview.h>

enum Widget { CheckBox=0, RadioButton, ColorFrame, ComboBox, EditBox, TextLabel };

class ListViewWidgets;

/** @short Klasa elementu listy widgetw (klasy ListViewWidgets).
 Zajmuje si ona rysowaniem niektrych widgetw. Obsuguje nastpujce widety:
 CheckBox, RadioButton, ColorFrame, ComboBox, EditBox, TextLabel. Umoliwia
 ustawianie oraz pobieranie wartoci poszczeglnych widgetw. Widget mona
 ustawi jako nieaktywny (dotyczy to tylko elementw typu TextLabel) lub
 aktywny. Rysowany jest rwnie tutaj kursor dla listy w postaci dwukolorowej
 ramki.
 */
class ListViewWidgetsItem : public QListViewItem
{
public:
	/** Konstruktor klasy.\n
	 @param parent - wskanik na rodzica,
	 @param w1 ... @param w8 - rodzaj widgetu w kolejnych komrkach.\n
	 Inicjowana jest tutaj tablica widgetw wartociami parametrw. Czyszczone s
	 te tablice wczonych widgetw o oraz tablica kolorw (dla widgetw
	 pokazujcych kolory).
	 */
	ListViewWidgetsItem( ListViewWidgets *parent,
	 Widget w1=TextLabel, Widget w2=TextLabel, Widget w3=TextLabel, Widget w4=TextLabel, Widget w5=TextLabel, Widget w6=TextLabel, Widget w7=TextLabel, Widget w8=TextLabel
	);

	/** Destruktor.\n
	 Brak definicji.
	 */
	virtual ~ListViewWidgetsItem() {}

	/** Metoda jest odpowiedzialna ze rysowanie komrki elementu.\n
	 @param p - wskanik 'rysownika' zwizany z otwartym urzdzeniem rysujcym,
	 @param cg - grupa kolorw do uycia,
	 @param col - bieca kolumna, w ktrej nastpuje rysowanie,
	 @param width - szeroko biecej kolumny,
	 @param alignment - strona do ustawiania zawartoci rysowanego elementu,
	 (lewa,centrum,prawa).\n
	 W komrce takiej znale si moe CheckBox, RadioButton, ColorFrame.
	 W przypadku pozostaych widgetw rysowanie jest przekazywane do klasy
	 macierzystej (QListView).
	*/
	virtual void paintCell( QPainter *p, const QColorGroup & cg, int col, int widht, int alignment );

	/** Metoda odpowiedzialna za rysowanie kursora.\n
	 @param p - wskanik 'rysownika' zwizany z otwartym urzdzeniem rysujcym
	 @param cg - grupa kolorw do uycia,
	 @param r - geometria dla kursora.
	 Kursor jest rysowany na komrce ktrej numer zwraca metoda
	  @see ListViewWidgets::cursorForColumn()
	 */
	virtual void paintFocus( QPainter *p, const QColorGroup & cg, const QRect & r );

	/** Zwraca przycity, podany cig tekstowy, ktry jest dopasowany do
	 szerokoci podanego numeru kolumny listy.\n
	 @param str - cig tekstowy do przycicia,
	 @param column - numer kolumny, w ktrej znajduje si podany cig,
	 @param columnWidth - szeroko kolumny, w ktrej znajduje si podany cig,
	 @param fontWeight - TRUE jeli cig jest pisany fontem QFont::bold,
	 w przeciwnym razie FALSE.
	 */
	QString trimedString( const QString & str, const int & column, const int & columnWidth, const int & fontWeight );

	/** Pobiera warto tekstow z podanej kolumny.\n
	 @param column - kolumna, z ktrej naley pobra warto tekstow,
	 @return cig tekstowy.\n
	 Przy czym zwracana jest prawdziwa warto, tylko jeli w podanej kolumnie
	 znajduje si jeden z nastpujcych widgetw: TextLabel, ComboBox, EditBox.
	 */
	QString textValue( uint column ) const;

	/** Ustawia podany tekst w podanej kolumnie.\n
	 @param column - kolumna, w ktrej naley ustawi podany tekst
	 @param str - tekst do ustawienia.\n
	 Przy czym, tekst jest ustawiany tylko jeli w podanej kolumnie znajduje si
	 jeden z nastpujcych widgetw: TextLabel, EditBox, ComboBox
	 */
	void setTextValue( uint column, const QString & str );

	/** Pobiera logiczny status widgeta z podanej kolumny.\n
	 @param column - kolumna, z ktrej naley pobra warto logiczn
	 @return warto logiczna (TRUE lub FALSE).\n
	 Przy czym zwraca prawdziw warto, tylko jeli znajduje si w niej jeden
	 z nastpujcych widgetw: CheckBox, RadioButton
	 */
	bool boolValue( uint column ) const;

	/** Ustawia warto logiczn w podanej kolumnie.\n
	 @param column - kolumna, w ktrej naley ustawi podan warto logiczn
	 @param value - warto do ustawienia (TRUE lub FALSE).\n
	 Przy czym warto jest ustawiana tylko, jeli w podanej kolumnie znajduje si
	 jeden z nastpujcych widgetw: CheckBox, RadioButton.
	 */
	void setBoolValue( uint column, bool value );

	/** Pobiera kolor widgeta z podanej kolumny.
	 @param column - kolumna, z ktrej naley pobra warto QColor
	 @return kolor - zwracany kolor widgetu z podanej kolumny.\n
	 Przy czym zwraca prawdziw warto, tylko jeli znajduje si w niej widget
	 typu ColorFrame.
	 */
	QColor colorValue( uint column ) const;

	/** Ustawia kolor widgeta w podanej kolumnie.\n
	 @param column - kolumna, w ktrej naley ustawi podan warto typu QColor,
	 @param color - warto QColor do ustawienia.\n
	 Przy czym warto jest ustawiana tylko, jeli w podanej kolumnie znajduje si
	 widget typu ColorFrame.
	 */
	void setColorValue( uint column, const QColor & color );

	/** Wstawia w widgetcie dla podanej kolumny podan list cigw tekstowych.\n
	 @param column - kolumna, w ktrej naley wstawi list,
	 @param stringList - lista cigw tekstowych.\n
	 Przy czym lista jest wstawiana tylko jeli w podanej kolumnie znajduje si
	 widgetu typu ComboBox.
	 */
	void insertStringList( uint column, const QStringList & stringList );

	/** Zwraca list cigw tekstowych.\n
	 @param column - kolumna, z ktrej naley pobra list,
	 @return lista cigw tekstowych.\n
	 Przy czym zwraca prawdziw warto, tylko jeli znajduje si w niej widget
	 typu ComboBox.
	 */
	QStringList stringList( uint column ) const;

	/** Ustawia atrybuty dla tekstu w podanej kolumnie,
	 @param column - kolumna w ktrej znajduje si tekst
	 @param color - kolor tekstu
	 @param bold - okrela czy tekst ma by pogrubiony
	 @param italic - okrela czy tekst ma by pochylony \n
	 Atrybuty s ustawiane tylko jeli w podanej kolumnie znajduje si jeden
	 z nastpujcych widgetw: TextLabel, EditBox, ComboBox. \n
	 @b UWAGA. Atrybuty mona ustawi tylko w kolumnie, dla ktrej jest ustawiony
	 kursor (patrz: @see ListViewWidgets::setCursorForColumn() )
	*/
	void setTextAttributs( uint column, const QColor & color, bool bold=FALSE, bool italic=FALSE );

	/** Zwraca rodzaj widgetu z podanej kolumny.\n
	 @param column - numer kolumny,
	 @return rodzaj widgetu.
	 */
	Widget widget( uint column ) const { return mWidgetsTab[column]; }

	/** Ustawia w podanej kolumnie podany widget.\n
	 @param column - kolumna w ktrej trzeba ustawi widget
	 @param widget - widget do ustawienia (CheckBox, ColorFrame, ComboBox, EditBox,
	 RadioButton, TextLabel)
	 */
	void setWidget( uint column, Widget widget );

	/** Wcza lub wycza tryb exclusive.\n
	 @param exlusive - TRUE wcza tryb 'exlusive', FALSE wycza ten tryb dla podanych widgetw,\n
	 @param widget   - rodzaj widgetw, ktre powinny dziaa w trybie exlusive
	 (tryb ten jest moliwy tylko dla widgetw typu: 'CheckBox' i 'RadioButton')
	 */
	void setExclusiveButtons( bool exclusive, Widget widget );

	/** Zwraca status trybu 'exlcusive' dla widgetw typu CheckBox i RadioButton.\n
	 @return TRUE tryb exclusive jest wczony, dla FALSE wyczony.
	 */
	bool exclusiveButtons() const { return mExclusiveButtons; }

	/** Przecza widget w podanej kolumnie na aktywny lub nieaktywny.\n
	 @param column - kolumna, w ktrej znajduje si widget do zmiany statusu,
	 @param enable - TRUE oznacza tryb aktywny, w przeciwnym razie wczany jest
	 tryb nieaktywny.\n
	 Przy czym dotyczy to tylko widgetw typu: TextLabel, EditBox, ComboBox.
	 */
	void setEnableCell( uint column, bool enable=TRUE );

	/** Zwraca status aktywnoci widgeta w podanej kolumnie.\n
	 @param column - kolumna, w ktrej znajduje si widget do sprawdzenia,
	 @return TRUE - oznacza, e widget jest aktywny, dla FALSE jest nieaktywny.\n
	 Przy czym prawdziwa warto jest zwracana tylko dla widgetw typu: TextLabel,
	 EditBox, ComboBox.
	 */
	bool isEnableCell( uint column ) const { return (mColorsTab[column] == Qt::black); }

private:
	ListViewWidgets *mParentList;

	Widget mWidgetsTab[8];

	bool mCheckedTab[8];
	QColor mColorsTab[8];
	QStringList mComboBoxLists[8];

	Widget mExlusiveButton;
	bool mExclusiveButtons;

	bool mBold, mItalic;
	QColor mTextColor;//, mTextBgColor;

protected:
	/** Ustawiane s tutaj wartoci widgetow w tablicy stanow.\n
	 @param column - kolumna klinitego wdigeta.\n
	 Jeli widget w podanej kolumnie ma wczony tryb exclusive, wtedy zostaje
	 mu ustawiona w tablicy warto TRUE, natomiast pozostaym FALSE. Jeeli
	 nie ma waczenego tego trybu, wtedy jest pomijany.\n
	 */
	void setExclusivelyButton( uint column );

};


#include "listvieweditbox.h"

#include <qcombobox.h>


/** @short Klasa obsuguje list elementw typu ListViewWidgetsItem.
 W komrkach tego elementu mona wstawi nastpujce rodzaje widgetw:
  @li warto logiczna (z edycj): CheckBox, RadioButton (dziaaj tak samo,
   a rni si tylko wygldem)
  @li lista cigw tekstowych - ComboBox
  @li ramka z kolorem - ColorFrame
  @li linijka edycyjna - EditBox
  @li tekst - TextLabel.\n
 Element listy moe mie powysze widgety maksymalnie w omiu pierwszych
 komrkach. Widgetom CheckBox i RadioButton mona ustawi tryb pracy "exlusive"
 (domylnie wyczony). Kliknicie w ColorFrame pokazuje dialog wyboru koloru.
 Widget EditBox moe pracowa w dwch trybach. Mianowicie jako pobieracz skrtw
 klawiszowych oraz jako zwyka linijka edycyjna. Po klikniciu w ten widget
 pokazywane jest okienko edycyjne pracujce w jednym z powyszych trybw.
 Kliknicie w widget typu ComboBox powoduje pokazanie obiektu typu QComboBox.
 Widget ten mona zainicjowa list cigw tekstowych, ktra bdzie pokazywana
 przy jego rozwiniciu. Dostpna jest moliwo ustawienia kursora dla wybranej
 kolumny. Po klikniciu w interaktywny widget wysyany jest odpowiedni sygna.
 Mianowicie dla ColorFrame - "colorChanged()", dla CheckBox i RadioButton -
 "buttonClicked()", dla EditBox - "editBoxTextApplied()".
*/
class ListViewWidgets : public QListView
{
	Q_OBJECT
public:
	/** Konstruktor klasy.\n
	 @param parent - wskanik na rodzica,
	 @param name - nazwa dla obiektu.\n
	 Tworzone s tutaj obiekty klas ListViewEditBox oraz QComboBox,
	 a take czone niezbdne sygnay ze slotami i inicjowane skadowe.
	 */
	ListViewWidgets( QWidget *parent = 0, const char *name = 0);

	/** Destruktor.\n
	 Usuwane s tutaj obiekty klas ListViewEditBox oraz QComboBox.
	 */
	virtual ~ListViewWidgets();

// 	void addColumn( Widget widget );

	/** Zwraca drugi kolor ta listy.\n
	 @return kolor.
	 */
	QColor secondColorOfBg() const { return mSecondColorOfBg; }

	/** Metoda ustawia drugi kolor ta (to moe by w paski).\n
	 @param c - nowy kolor drugiego paska ta (pierwszy jest zawsze biay).
	 */
	void setSecondColorOfBg( const QColor & c );

	/** Zwraca informacj o tym czy to jest dwukolorowe.\n
	 @return TRUE - oznacza, e to jest dwukolorowe, w przeciwnym razie ma jeden.
	 */
	bool enableTwoColorsOfBg() const { return mEnableTwoColorsOfBg; }

	/** Wcza/wycza dwukolorowe to.\n
	 @param TRUE - wymusza rysowanie dwukolorowego to, w przeciwnym razie bdzie
	 mie jeden kolor.\n
	 Domylnie waciwo ta jest wyczona.
	 */
	void setEnableTwoColorsOfBg( bool enable = TRUE );

//		QColor insideFrameCursorColor() const { return mInsideFrameCursorColor; }
// 	void setInsideFrameCursorColor( const QColor & color ) {
// 		mInsideFrameCursorColor = color;  triggerUpdate();
// 	}
//		QColor outsideFrameCursorColor() const { return mOutsideFrameCursorColor; }
// 	void setOutsideFrameCursorColor( const QColor & color ) {
// 		mOutsideFrameCursorColor = color; triggerUpdate();
// 	}

	/** Pokazuje widget w podanej kolumnie na pozycji podanego elementu.\n
	 @param column - kolumna, w ktrej naley pokaza widget,
	 @param item - wskanik na element, na ktrym naley pokaza widget.
	 Metoda obsuguje widgety: ComboBox i EditBox.
	 */
	void showWidget( int column, ListViewWidgetsItem *itemPtr=0 );

	/** Chowa widget w podanej kolumnie na pozycji biecego elementu, tylko jeli
	 znajduje si w niej ComboBox lub EditBox.\n
	 @param column - kolumna w ktrej naley ukry widget.
	 */
	void hideWidgetForCurrentItem( int column );

	/** Ustawia numer kolumny, w ktrej bdzie pokazywany kursor.\n
	 @param column - numer kolumny.
	*/
	void setCursorForColumn( uint column ) {
		if ( column <= (uint)columns() )  mCursorForColumn = column;
	}

	/** Zwraca numer kolumny, ktrej jest pokazywany kursor.\n
	 @return numer kolumny.
	*/
	uint cursorForColumn() const { return mCursorForColumn; }

	/** Zwraca status pracy widgetu typu EditBox jako pobieracz kodw klawiszy.\n
	 @return TRUE widget EditBox pracuje jako pobieracz kodw klawiszy, FALSE
	  oznacza e widget pracuje jako zwyka linijka edycyjna.
	 */
	bool editBoxGetKeyShortcutMode() const { return mEditBoxGetKeyShortcut; }

	/** Ustawia tryb pracy widgetu EditBox jako jako pobieracz kodw klawiszy.\n
	 @param editBoxGetKeyShortcutMode - rwne TRUE oznacza, e EditBox bdzie
	 pobiera kody klawiszy, w przeciwnym razie bdzie linijk edycjn.
	 */
	void setEditBoxGetKeyShortcutMode( bool editBoxGetKeyShortcut ) { mEditBoxGetKeyShortcut = editBoxGetKeyShortcut; }

private slots:
	/** Slot obsuguje kliknicie w komrk elementu listy.\n
	 Jeli w podanej kolumnie znajduje si jeden z podanych widgetw: ComboBox,
	 EditBox, wtedy jest on pokazywany. Jeli w podanej kolumnie znajduje si
	 jeden z podanych widgetw: CheckBox, RadioButton, wtedy nastpuje zmiana jego
	 stanu. Jeli kolumna zawiera widget ColorFrame, wtedy pokazywany jest dialog
	 wyboru koloru. \n
	 @param item - wskanik kliknitego elementu
	 @param vp - globalna pozycja kursora w momencie kliknicia
	 @param column - kolumna, w ktrej kliknito
	 */
	void slotClicked( QListViewItem * item, const QPoint & point, int column );

	/** Slot wywoywany w momencie zmiany rozmiaru kolumn.\n
	 Jeli w kolumnie znajduje si widget (widoczny): ComboBox lub EditBox, wtedy
	 zmieniany jest jego rozmiar (i ewentualnie pozycja).\n
	 @param section - kolumna, ktra ma zmieniany rozmiar
	 @param oldSize - szeroko kolumny sprzed zmiany rozmiaru
	 @param newSize - szeroko kolumny po zmianie rozmiaru
	 */
	void slotHeaderSizeChange( int section, int oldSize, int newSize );

	/** Ustawia jako biecy numer elementu dla widgeta typu ComboBox numer
	 podanego elementu.\n
	 @param column - kolumna, w ktrej znajduje si widget ComboBox,
	 @param id - numer elementu, ktry ma by aktywny.
	 */
	void slotSetCurrentComboItem( int id );

private:
	bool mEditBoxGetKeyShortcut;
	uint mCursorForColumn;
	int mCurrentColumn;
	int mEditColumn;

	bool mEnableTwoColorsOfBg;
	QColor mSecondColorOfBg;
	//QColor mInsideFrameCursorColor,  mOutsideFrameCursorColor;

	ListViewEditBox *mEditBox;
	QComboBox *mComboBox;

signals:
	/** Sygna wysyany gdy zostanie zmieniony kolor w widgetcie ColorFrame
	 w podanej kolumnie.\n
	 @param column - kolumna, w ktrej znajduje si wymagany widget.
	 */
	void colorChanged( int column );

	/** Sygna wysyany w momencie, gdy zostanie kliknity widget typu CheckBox
	 lub RadioButton w podanej kolumnie.\n
	 @param column - kolumna, w ktrej znajduje si wymagany widget.
	 */
	void buttonClicked( int column );

	/** Sygna wysyany, gdy zostanie zmieniona zawarto widgetu EditBox.\n
	 @param column - kolumna, w ktrej znajduje si wymagany widget,
	 @param oldText - poprzedni tekst z podanej kolumny,
	 @param newText - nowy tekst dla podanej kolumny.
	 */
	void editBoxTextApplied( int column, const QString & oldText, const QString & newText );

};

#endif
