/***************************************************************************
                          locationchooser.h  -  description
                             -------------------
    begin                : wed mar 17 2004
    copyright            : (C) 2004 by Piotr Mierzwiski
    email                : peterm@go2.pl

    copyright            : See COPYING file that comes with this project
 ***************************************************************************/

#ifndef LOCATIONCHOOSER_H
#define LOCATIONCHOOSER_H

#include <qlayout.h>
#include <qwidget.h>
#include <qlineedit.h>
#include <qcombobox.h>
#include <qpopupmenu.h>
#include <qtoolbutton.h>


/**
 * Author:
 * Piotr Mierzwiski
 **/
/** @short Klasa wspomagajca wybr pliku lub katalogu.
 Obiekt tej klasy skada si z widgetu pobierajcego (linijka edycyjna lub
 lista typu 'ComboBox') oraz guzika z menu, ktre zawiera opcje zwizane z
 widgetem pobierajcym. Kady element w menu mona ukry i pokaza jak rwnie
 uczyni nieaktywnym lub aktywnym. Po wstawieniu cieki katalogu wysyany jest
 sygna: @em locationChanged() . Wygld widgetu mona modyfikowa za pomoc
 dostpnych metod. Moliwa jest zmiana widgetu pobierajcego na linijk
 edycyjn lub list ComboBox, guzik z menu moe by umieszczony z lewej, lub
 z prawej strony widgetu pobierajcego, moliwe jest rwnie jego usunicie.
 Lista cieek z widetu ComboBox moe by automatycznie odczytywana oraz
 zapisywana do pliku konfiguracyjnyego (poprzez metod), jeli ustawiony
 zostanie dla niego klucz.
 */
class LocationChooser : public QWidget
{
	Q_OBJECT
public:
	/** Tryb pobierania scieki (dla pliku albo dla katalogu).
	 */
	enum GetMode { GetDirectory=0, GetFile };

	/** Identyfikatory akcji w menu guzika.
	 */
	enum MenuItems { CURRENT_DIR=0, ORIGINAL_DIR, HOME_DIR, ROOT_DIR, PREVIOUS_DIR, NEXT_DIR, SELECT_DIR, CLEAR_CURRENT, CLEAR_ALL, SET_DEFAULT_ACTION };
	/** Rodzaj obiektu pokazywanego obok guzika z menu.
	 */
	enum GettingWidget { LineEdit=0, ComboBox };

	/** Rodzaj strony, na ktrej bdzie pokazany guzik, lub jego brak.
	 */
	enum ButtonSide { LeftButton=0, RightButton, NoneButton };


	/** Konstruktor klasy.\n
	 @param parent - wskanik na rodzica,
	 @param name - nazwa dla obiektu.\n
	 Inicjowane s tutaj skadowe oraz tworzone obiekty skadajce si na biecy.
	 */
	LocationChooser( QWidget *parent, const char *name=0 );

	/** Destruktor.\n
	 Usuwane s tu obiekty uywane w klasie.
	 */
	~LocationChooser();

	/** Ustawia biec ciek aplikacji.\n
	 @param currentPath - nazwa biecej cieki.
	 */
	void setCurrentPath( const QString & currentPath )   { mCurrentPath  = currentPath;  }

	/** Ustawia ciek, ktra bya biec.\n
	 @param originalPath - poprzednia bieca cieka.
	 */
	void setOriginalPath( const QString & originalPath ) { mOriginalPath = originalPath; }

	/** Ukrywa bd pokazuje element o podanym Id w menu widoku.\n
	 @param id - identyfikator elementu menu (patrz @see MenuItems),
	 @param visible - TRUE, oznacza e element bdzie widoczny, FALSE powoduje
	  jego ukrycie.
	 */
	void setMenuItemVisible( int id, bool visible );

	/** Uaktywnia lub dezaktywuje element o podanym Id w menu widoku.\n
	 @param id - identyfikator elementu menu (patrz @see MenuItems),
	 @param enable - TRUE, oznacza e element bdzie aktywny, FALSE powoduje
	  jego dezaktywacje.
	 */
	void setMenuItemEnabled( int id, bool enable );

	/** Ustawia focus na widgetcie pobierania cieki (QLineEdit).
	 */
	void setFocus() { mLineEdit ? mLineEdit->setFocus() : mComboBox->setFocus(); }

	/** Metoda ustawia nazw klucza (dla pliku konfiguracyjnego), z ktrego
	 naley odczyta list URLi.\n
	 @param keyReadingStr - pena nazwa klucza.
	 */
	void setKeyListReading( const QString & keyListReadingStr, bool readSettings=FALSE );

	/** Powoduje zaznaczenie biecej zawartoci widgetu pobierajcego.
	 */
	void selectAll();

	/** Powoduje wstawienia podanego elementu na podanej pozycji na list
	 ComboBox.\n
	 @param itemStr - cig tekstowy do wstawienia,
	 @param itemId - pozycja, na ktrej naley umieci cig.\n
	 Metoda obsuguje obiekt pobierania ComboBox, jeli aktualnym jest LineEdit,
	 wtedy poday cig jest ustawiany na nim. Jeli podany cig ju istnieje na
	 licie, wtedy nie jest on wstawiany.
	 */
	void insertItem( const QString & itemStr, uint itemId=0 );

	/** Wstawiana jest tu na list ComboBox podana lista cigw tekstowych.\n
	 @param stringList - lista do wstawienia.
	 */
	void insertStringList( const QStringList & stringList );

	/** Metoda powoduje wczytanie listy elementw na list.\n
	 Przy czym jest robione tylko jeli obiektem pobierajcym jest ComboBox oraz
	 nazwa klucza dla pliku konfiguracyjnego nie jest pusta.
	 \n \n Patrz te: @see setKeyListReading.
	 */
	void init();

	/** Zapisuje do pliku konfiguracyjnego list obiektu pobierajcego ComboBox.\n
	 Przy czym korzysta z wczeniej ustawionego klucza, jeli jest on pusty to
	 nic nie jest robione.
	 \n \n Patrz te: @see setKeyListReading.
	 */
	void saveSettings();

	/** Zwraca nastpny wzgldem biecego element listy.\n
	 @return nastpny element listy ComboBox.\n
	 Przy czym, jeli biecy jest ostatnim, wtedy zwracany jest pierwszy.
	 */
	QString nextEntry() const;

	/** Zwraca poprzedni wzgldem biecego element listy.\n
	 @return poprzedni element listy ComboBox.
	 Przy czym, jeli biecy jest pierwszym, wtedy zwracany jest ostatni.
	 */
	QString prevEntry() const;

	/** Zwraca aktualn zawarto obiektu pobierajcego.\n
	 @return cig tekstowy
	 */
	QString text( int id=-1 ) const;

	/** Zwraca informacj o rodzaju pobierania (plik/katalog).\n
	 @return rodzaj pobierania, patrz @see GetMode.\n
	 Waciwo domylnie ustawiona jest na 'GetDirectory'.
	 */
	GetMode getMode() const { return mGetMode; }

	/** Zwraca informacj o tym czy pokazywany jest separator pomidzy widgetem
	 pobierajcym a guzikiem z menu.\n
	 @return TRUE - separator jest pokazywany, FALSE - nie jest.\n
	 Domylnie separator jest pokazywany.
	 */
	bool showSeparator() const { return mShowSeparator; }

	/** Zwraca informacj o tym czy pokazywane bd w menu (dla widgetu ComboBox)
	 opcje: "Previous directory" i "Next directory".\n
	 @return TRUE - opcje s pokazywane, FALSE - s ukrywane.\n
	 Domylnie opcje te s ukrywane.
	 */
	bool showPrevNextActions() const { return mShowPrevNextActions; }

	/** Zwraca informacj o tym z ktrej strony jest pokazywany guzik z menu,
	 albo czy nie jest pokazywany.\n
	 @return strona pokazywania, patrz te: @see ButtonSide.
	 */
	ButtonSide buttonSide() const { return mButtonSide; }

	/** Zwraca informacj o rodzaju widgetu pobierajcego.\n
	 @return identyfikator widgetu, patrz te: @see GettingWidget.
	 */
	GettingWidget gettingWidget() const { return mGettingWidget; }

public slots:
	/** Ustawia podany tekst jako biecy widgetcie pobierajcym.\n
	 @param text - tekst do ustawienia.
	 */
	void setText( const QString & text );

	/** Ustawia tryb pobierania cieki przez dialog pokazywany po wybraniu opcji
	 "Select directory" z menu guzika.\n
	 @param mode - tryb pobierania (patrz: @see GetMode).\n
	 */
	void setGetMode( GetMode mode ) { mGetMode = mode; }

	/** Ustawia stron pokazywania guzika lub usuwa go.\n
	 @param btnSide - strona dla guzika lub wymuszenie usunicia go.\n
	 Po wywoaniu tej metody widget jest przebudowywany.
	 */
	void setButtonSide( ButtonSide btnSide );

	/** Metoda powoduje pokazanie lub ukrycie separatora pomidzy widgetem
	 pobierajcym a guzikiem z menu.\n
	 @param show - rwne TRUE, spowoduje pokazanie separatora, w przeciwnym razie
	 zostanie on ukryty.
	 */
	void setShowSeparator( bool show );

	/** Umoliwia zmian widgetu pobierajcego na podany.\n
	 @param getWidget - typ nowego widgetu, patrz te: @see GettingWidget \n
	 Po wywoaniu tej metody widget jest przebudowywany.
	 */
	void setGettingWidget( GettingWidget getWidget );

	/** Metoda powoduje e dla widgetu ComboBox zawsze bd pokazywane lub
	 ukrywane opcje z menu "Previous directory" i "Next directory".\n
	 @param showPrevNext - rwne TRUE, oznacza e opcje bd pokazywane, FALSE
	 mwi e naley je ukrywa.
	 */
	void setShowPrevNextActions( bool showPrevNext ) { mShowPrevNextActions = showPrevNext; }

signals:
	/** Sygna informuje o zmianie cieki w widgetcie pobierajcym.\n
	 @param filePath - nowa cieka w widoku.\n
	 Sygna jest wysyany po wstawieniu cieki do widoku.
	 */
	void locationChanged( const QString & filePath );

	/** Sygna wysyany w momencie zmiany zawartoci linijki edycyjnej.\n
	 @param text - nowy tekst
	 */
	void textChanged( const QString & text );

private slots:
	/** Slot uywany przez menu guzika przy wybieraniu opcji.\n
	 @param itemId - identyfikator opcji, patrz @see MenuItems.
	 */
	void slotSelectDirMenuItem( int itemId );

	/** Slot pokazuje dialog pobierania cieki do pliku lub katalogu.\n
	 To czy bdzie wybierany plik czy katalog zaley od ustawienia skadowej
	 @em mGetMode . Patrz te: @see setGetMode().
	 */
	void slotGetExistingDirectory();

private:
	QString mCurrentPath, mOriginalPath;

	GetMode mGetMode;
	ButtonSide mButtonSide;
	GettingWidget mGettingWidget;
	bool mShowSeparator, mShowPrevNextActions;

	QHBoxLayout *mLayout;
	QComboBox   *mComboBox;
	QLineEdit   *mLineEdit;
	QToolButton *mChooserButton;
	QPopupMenu  *mButtonMenu;

	QString mKeyListReadingStr;

	/** Metoda powoduje budowanie widgetu skadajcego si z guzika oraz widgetu
	 pobierajcego.\n
	 Wykonywane jest to na podstawie skadowych klasy okrelajcych wygld.
	 */
	void createWidget();

	/** Metoda powoduje utworzenie guzika z menu.
	 */
	void createChooserButton();

};

#endif
