/***************************************************************************
                          statusbar.h  -  description
                             -------------------
    begin                : sun oct 27 2002
    copyright            : (C) 2002 by Piotr Mierzwiski
    email                : peterm@go2.pl

    copyright            : See COPYING file that comes with this project
 ***************************************************************************/

#ifndef _STATUSBAR_H_
#define _STATUSBAR_H_

#include "squeezelabel.h"

#include <qprogressbar.h>

/**
 * Author:
 * Piotr Mierzwiski
 **/
/**
 @short Klasa odpowiedzialna za obsug paska statusu.
 Dziki dziedziczeniu po klasie SqueezeLabel informacja na pasku moe by
 ciskana. Poza tym obsugiwane jest tutaj dostosowywanie dugoci informacji
 dla panela plikw zalene od szerokoci paska. Dostpne s trzy tryby
 pokazywania, tj.: may (SmallMSG), redni (TinyMSG) oraz duy (BigMSG).
 Przeczanie pomidzy trybami jest wykonywane automatycznie przy zmianie
 szerokoci paska. Dodatkowo klasa wspiera pokazywanie paska postpu, ktry
 przykrywa cay pasek statusu. Wspierane jest take czasowe pokazywanie
 informacji i po jego miniciu przywracany jest poprzedni lub nastpuje
 czyszczenie widoku.
 */
class StatusBar : public SqueezeLabel
{
	Q_OBJECT
public:
	/** Konstruktor klasy.\n
	 @param parent - wskanik na rodzica,
	 @param name - nazwa dla obiektu.\n
	 Nastpuje tutaj inicjowanie obiektu klasy oraz utworzenie obiektu
	 paska postpu.
	 */
	StatusBar( QWidget *parent, const char *name=0 );

	/** Destruktor.\n
	 Usuwany jest tutaj obiekt paska postpu.
	 */
	~StatusBar();

	/** Metoda czyci pasek statusu lub chowa pasek postpu.\n
	 @param progress - TRUE wymusza schowanie paska postpu, w przeciwnym
	 razie czyszczona jest zawrto paska statusu.
	 */
	void clear( bool progress=FALSE );

	/** Wcza/wycza ciskanie zawartoci paska.\n
	 @param squeezing - TRUE wcza ciskanie, FALSE wycza je.
	 */
	void setSqueezingText( bool squeezing ) { setScaledContents( squeezing ); }

private:
	QProgressBar *mProgress;

	uint mHtmlTagsWidth, mWidthOfListStatusStr[3];
	uint mNumOfAllItems, mNumOfSelectedItems, mWeightOfSelected;
	QString mStatusStr, mOldStatusStr, mListStatusStr;
	bool mShowListStatus, mShowPrevMsg;

	/** Rodzaj trybu pokazywania informacji dla panela plikw.
	 */
	enum SizeOfMessage { SmallMSG, TinyMSG, BigMSG };
	SizeOfMessage mSizeOfMessage;

protected:
	/** Inicjuje skadow zawierajc informacje do pokazania na pasku o dugoci,
	 na ktr wskazuje tryb pokazywania (patrz @see SizeOfMessage).\n
	 Powoduje przypisnie do skadowej @em mListStatusStr wskazywanego przez
	 @em mSizeOfMessage' odpowiedniej dugoci cigu informacyjnego, ktry zawiera
	 informacje o iloci wszystkich elementw na licie, iloci zaznaczonych, oraz
	 wadze wszystkich zaznaczonych plikw. Pena informacja wyglda nastpujca:
	 "Zaznaczono X plikw z Y, o cakowitym rozmiarze Z bajtw", skrcona:
	 "X plikw z Y, rozmiar Z", a najkrtsza tak: "X, Y, Z".
	 */
	void setStatusListStr();

	/** Ustawiana jest tutaj skadowa zawierajca informacj o trybie pokazywania
	 informacji na pasku.\n
	 Po ustawieniu skadowej wywoywana jest funkcja @em setStatusListStr .
	 */
	void squeezeListStatus();

	/** Obsuga zdarzenie zmiany rozmiaru.\n
	 Pasek postpu dostaje biec szeroko paska, wywoywana jest tutaj rwnie
	 funkcja ustawiajca tryb pokazywania informacji, po czym jest ona wywietlana
	 */
	void resizeEvent( QResizeEvent * );

public slots:
	/** W slocie jest uaktualniana warto paska statusu.\n
	 @param value - procentowa warto postpu.\n
	 Jeli pasek postpu nie jest widoczny to jest pokazywany, podanie wartoci
	 101 spowoduje ukrycie go, tak samo si dzieje po osigniciu wartoci 100
	 przez pasek postpu.
	 */
	void slotSetProgress( int value );

	/** Ustawia podany cig na pasku i ewentualnie ukrywa po miniciu okrelonego
	 czasu.\n
	 @param str - informacja do ustawienia,
	 @param ms - liczba milisekund po ktrym nastpi ukrycie podanej informacji,
	 @param showPrevious - TRUE wymusza pokazanie poprzedniej zawartoci paska,
	  FALSE powoduje poprostu wyczyszczenie go, kiedy minie podany czas.
	 */
	void slotSetMessage( const QString & str, uint ms=0, bool showPrevious=TRUE );

	/** Slot powoduje uaktualnianie statusu dla panela plikw informacj o iloci
	 i wadze jego plikw.\n
	 @param numOfSelectedItems - liczba zaznaczonych elementw,
	 @param numOfAllItems - liczba wszystkich elementw,
	 @param weightOfSelected - waga wszystkich zaznaczonych plikw.
	 */
	void slotUpdateOfListStatus( uint numOfSelectedItems, uint numOfAllItems, uint weightOfSelected );

private slots:
	/** Slot przywraca poprzedni informacje lub czyci widok.\n
	 ( w zalenoci od wartoci skadowej @em mShowPrevMsg ustawianej w
	 @em slotSetMessage() )
	 */
	void slotTimerDone();

};

#endif
