/***************************************************************************
                          filesassociation.h  -  description
                             -------------------
    begin                : wed may 12 2004
    copyright            : (C) 2004 by Piotr Mierzwiski
    email                : peterm@go2.pl

    copyright            : See COPYING file that comes with this project
 ***************************************************************************/

#ifndef _FILESASSOCIATION_H_
#define _FILESASSOCIATION_H_

#include "../../enums.h"

#include <qvaluelist.h>

/**
 * Author:
 * Piotr Mierzwiski
 **/
/**
 @short Klasa obsuguje skojarzenia dla plikw.
 W klasie tworzona jest lista skojarze plikw poprzez zaadowanie ustawie
 z pliku konfiguracyjnego (qtcmd_mimerc). Jeli plik taki nie zostanie
 znaleziony na list wstawione zostan domylne wartoci dla skojarze. Mona
 pobra nastpujce informacje z obiektu tej klasy:
  @li czy skojarzenia zostay zaadowane,
  @li nazwa aplikacji dla podanego pliku,
  @li rodzaj pliku dla podanego, patrz: @see KindOfFile,
  @li wzorce dopasowywa dla podanego typu pliku,
  @li rodzaj podgldu (wewntrzny/zewntrzny), patrz: @see KindOfViewer,
*/
class FilesAssociation
{
public:
	/** Konstruktor klasy.\n
	 adowany jest tutaj plik skojarze i na jego podstawie tworzona lista
	 skojarze. W przypadku brak pliku, lista jest uzupeniana domylnymi
	 wartociami.
	 */
	FilesAssociation();

	/** Destruktor.\n
	 Brak definicji (jest pusta).
	 */
	~FilesAssociation() {}


	/** Zwraca informacj o tym czy skojarzenia zostay zaadowane.\n
	 @return TRUE, jeli zaadowano skojaenia, FALSE - nie zostao to zrobione.
	 */
	bool loaded() const { return mFilesAssociationLoaded; }


	/** Zwraca informacj o nazwie przegldarki dla podanego pliku.\n
	 @param fileName - nazwa pliku do sprawdzenia,
	 @return nazwa przegldarki z absolutn ciek.
	 */
	QString appsForView( const QString & fileName );

	/** Zwraca informacj o rodzaju pliku dla podanego.\n
	 @param fileName - nazwa pliku do sprawdzenia,
	 @return rodzaj pliku, patrz: @see KindOfFile.
	 */
	KindOfFile kindOfFile( const QString & fileName );

	/** Zwraca wzorce dopasowa dla podanego typu pliku.\n
	 @param fileType - typ pliku (avi/gif/mp3),
	 @return wzorce dopasowa (rozszerzenia plikw, np. *.ogg).
	 */
	QString fileTemplates( const QString & fileType );

	/** Zwraca informacj o rodzaju podgldu (zewntrzny/wewntrzny) dla podanego.\n
	 @param fileName - nazwa pliku do sprawdzenia,
	 @return rodzaj podgldu, patrz: @see KindOfViewer.
	 */
	KindOfViewer kindOfViewer( const QString & fileName );

	/** Zwraca opis typu dla podanego pliku.\n
	 @param fileName - nazwa pliku do sprawdzenia,
	 @return opis typu.
	 */
	QString fileDescription( const QString & fileName );

private:
	bool mFilesAssociationLoaded;


	/** Struktura opisujca skojaenie dla pliku.\n
	 @param fileType - typ pliku (gif/avi itp.);
	 @param appsForView - absolutna nazwa przegldarki dla biecego typu;
	 @param templates - wzorce dopasowywa (rozszerzenia plikw, np.: *.avi);
	 @param kindOfViewer - rodzaj podgldu, patrz: @see KindOfViewer;
	 @param kindOfFile - rodzaj pliku, patrz: @see KindOfFile.
	 */
	struct FileAssociation {
		QString fileType;
		QString appsForView;
		QString description;
		QString templates;
		KindOfViewer kindOfViewer;
		KindOfFile kindOfFile;
	};
	typedef QValueList <FileAssociation> FilesAssociationList;
	typedef FilesAssociationList::iterator Iterator;
	FilesAssociationList mFilesAssociationList;


	/** Zwraca identyfikator rodzaju pliku dla podanej nazwy typu pliku.\n
	 @param kindOfFileStr - nazwa rodzaju pliku, obsugiwane s nastpujce:
	 (Source, RenderableText, Image, Sound, Video),
	 @return rodzaj pliku, patrz: @see KindOfFile.\n
	 Metoda uywana podczas inicjowania listy w konstruktorze.
	 */
	KindOfFile kindOfFileFromName( const QString & kindOfFileStr );

	/** Zwraca pozycj znalezionego dopasowania podanego pliku na licie
	 skojarze.\n
	 @param fileName - nazwa pliku do dopasowania,
	 @return pozycja na licie, lub warto mFilesAssociationList.end(), jeli
	  nie odnaleziono dopasowania.\n
	 Metoda uywana jest do znajdowania przegldarki dla podanego pliku, rodzaju
	 przegldarki, rodzaju podanego pliku, w metodach za to odpowiedzialnych.
	 */
	Iterator find( const QString & fileName );

};

#endif
