/***************************************************************************
                          pathview.h  -  description
                             -------------------
    begin                : wed oct 23 2002
    copyright            : (C) 2002 by Piotr Mierzwiski
    email                : peterm@go2.pl

    copyright            : See COPYING file that comes with this project
 ***************************************************************************/


#ifndef _PATHVIEW_H_
#define _PATHVIEW_H_

#include "combopath.h"
#include "enums.h"

#include <qwidget.h>
#include <qlistview.h>
#include <qpopupmenu.h>
#include <qtoolbutton.h>

/**
 * Author:
 * Piotr Mierzwiski
 **/
/** @short Klasa pokazujca list ze ciekami oraz guzik z dodatkowymi opcjami.
 W klasie tworzony jest obiekt listy cieek ( @em ComboPath ), ponad nastpuje
 utworzenie guzika z opcjami powodujcymi zmian biecej cieki na wybran
 oraz opcjami edycji historii cieek (czyszczenie i edycja waciwa).\n
 Poza tym klasa zawiera metody poredniczce w obsudze widoku cieek, takie
 jak: wstawianie, usuwanie podanej cieki, ustawianie jako biecej cieki tej
 o podanym numerze a take pokazywanie historii cieek.\n
 Dodatkow cech obiektu tej klasy jest zmiana ta widoku w zalenoci od
 statusu zapisywalnoci biecego katalogu przez aktualnie zalogowanego
 uytkownika. To jest blado zielone jeli mona zapisywa, blado czerwone,
 jeeli brak tej moliwoci oraz biae gdy nie jest znany ten status.
*/
class PathView : public QWidget
{
	Q_OBJECT
public:
	/** Konstruktor klasy.\n
	 @param ptrOfList - wskanik na list plikw,
	 @param parent - wskanik na obiekt rodzica,
	 @param name - nazwa dla obiektu.\n
	 Inicjowane s tutaj niezbdne skadowe oraz tworzone obiekty klasy ComboPath
	 oraz QToolButton (przycisk z menu).
	 */
	PathView( QListView *ptrOfList, QWidget *parent, const char *name=0 );

	/** Destruktor klasy.\n
	 Nastpuje tutaj zapisanie do pliku konfiguracyjnego listy URLi, oraz id dla
	 biecego, poza tym usuwane s obiekty tworzone w klasie.
	 */
	~PathView();

	/** Zwraca biec ciek.\n
	 @return biecy cig na licie.\n
	 */
	QString currentUrl() const { return mComboPathView->currentText(); }

	/** Zwraca indeks biecego elementu na licie @em ComboPath .
	 @return indeks biecego elementu.
	 */
	int currentId() const { return mComboPathView->currentItem(); }

	/** Ustawia focus na widoku cieki ( obiekt @em ComboPath ).
	 */
	void setFocus() { mComboPathView->setFocus(); }

	/** Zwraca informacj o obecnoci focusa na widoku cieki.\n
	 @return TRUE, jeli widok cieki zawiera focus, FALSE widok go nie posiada.
	 */
	bool hasFocus() const { return mComboPathView->hasFocus(); }

	/** Rozwija list cieek z odwiedzonymi katalogami.
	 */
	void showPopupList();

	/** Powoduje pokazanie menu guzika znajdujcego si obok widoku listy cieek.
	 */
	void showButtonMenu();

	/** Zwraca poprzedni wzgldem biecego element listy.\n
	 @param localFS - rwny TRUE, wtedy szukana jest cieka z lokalnego systemu
	 plikw, w przeciwnym razie zwrcona zostanie pierwsza napotkana.\n
	 @return poprzedni URL.\n
	 Przy czym, jeli biecy URL jest pierwszym, wtedy zwracany jest ostatni.
	 */
	QString prevUrl( bool localFS=FALSE ) const;

	/** Zwraca nastpny wzgldem biecego element listy.\n
	 @return nastpny URL.\n
	 Przy czym, jeli biecy URL jest ostatnim, wtedy zwracany jest pierwszy.
	 */
	QString nextUrl() const {
		return mComboPathView->text( mComboPathView->nextItemId() );
	}

private:
	QToolButton *mButtonQuickCD;
	ComboPath   *mComboPathView;
	QPopupMenu  *mQuickCdMenu;


	/** Ustawia kolor ta widoku cieki na blado zielony lub blado czerwony.\n
	 Przed ustawieniem koloru nastpuje sprawdzenie statusu biecej cieki.
	*/
	void setStatusWritable();

	/** Metoda inicjujca widok cieki.\n
	 Wczytywana jest tutaj i wstawiana do widoku, lista URLi oraz numer biecego
	 URLa, ktry staje si biecym w widoku.
	 */
	void init();

	/** Wywouje sygna uruchamiajcy zmian biecej cieki w systemie plikw.\n
	 @param newPath - nowa cieka.
	 */
	void chagePathInFS( const QString & newPath );

	/**
	 */
	void updateToolTips();

protected:
	/** Obsugiwane jest tutaj zdarzenie zmiany rozmiaru dla biecego obiektu.\n
	 Ustawiana jest odpowiednia szeroko widoku cieki (pomniejszona o warto
	 28, ktra jest szerokoci guzika z menu).
	 */
	void resizeEvent( QResizeEvent * );

public slots:
	/** Powoduje wstawienie na list nowego elementu (cieki).\n
	 @param newPath - nazwa elementu do usunicia.\n
	 Przy czym element jest wstawiany, tylko jeli previousURL sam nie istnieje ju na
	 licie.
	 */
	void slotPathChanged( const QString & newPath );

	/** Powoduje usunicie podanego elementu (cieki) z listy.\n
	 @param pathName - nazwa elementu do usunicia.\n
	 Przy czym jako biecy ustawiany jest poprzedni, a jeli nie istnieje
	 poprzedni, wtedy brany jest nastpny. Jeli lista zawiera 1 lub 0 elementw,
	 wtedy nic nie jest robione.
	 */
	void slotRemovePath( const QString & pathName );

private slots:
	/** Uruchamia zmian katalogu na wyszy w hierarchi (w biecym systemie
	 plikw).\n
	 Wysyany jest tutaj tylko sygna wymuszajcy powysz zmian.
	 */
	void slotChangeDirToUp();

	/** Powoduje zmian katalogu na korze (w biecym systemie plikw).
	 */
	void slotChangeDirToRoot();

	/** Powoduje zmian katalogu na domowy w lokanym systemie plikw.
	 */
	void slotChangeDirToUserHome();

	/** Powoduje zmian katalogu na poprzedni wzgldem biecego (z historii).
	 */
	void slotChangeDirToPrev();

	/** Powoduje zmian katalogu na nastpny wzgldem biecego (z historii).\n
	 Przy czym jeli nie ma poprzedniego to brany jest ostatni.
	 */
	void slotChangeDirToNext();

	/** Usuwa wszystkie wpisy na licie historii cieek.
	 */
	void slotClearHistoryPath();

	/** Powoduje wyczyszczenie biecej cieki.
	 */
	void slotClearPath();

	/** Pokazuje okienko suce edycji historii cieek.\n
	 Pozwala ono na: usunicie, edycj oraz zmian kolejnoci na licie.
	 */
	void slotEditHistory();

signals:
	/** Sygna powoduje pobranie statusu zapisywalnoci biecego katalogu przez
	 aktualnie zalogowanego uytkownika.
	 @param dw - zmienna, ktrej zapisana zostanie informacja, patrz te:
	  @see DirWritable.\n
	 */
	void signalPossibleDirWritable( DirWritable & dw );

	/** Sygna uruchamia zmian biecego katalogu na podany.
	 @param newPath - nowa cieka do odczytania.
	 */
	void signalPathChanged( const QString & newPath );

};

#endif
