/***************************************************************************
                          popupmenu.h  -  description
                             -------------------
    begin                : Tue Aug 21 2001
    copyright            : (C) 2002, 2003 by Piotr Mierzwiski
    email                : peterm@go2.pl

    copyright            : See COPYING file that comes with this project
 ***************************************************************************/

#ifndef POPUPMENU_H
#define POPUPMENU_H

#include "enums.h"

#include <qwidget.h>
#include <qlistview.h>
#include <qpopupmenu.h>


#define CMD_Separator          1
#define CMD_SubMenu            1000

// menu for file list/tree
#define CMD_Open               101
#define CMD_OpenInPanelBeside  102
#define CMD_Rename             103
#define CMD_Delete             104
#define CMD_Copy               105
#define CMD_MakeLink           106
#define CMD_Move               107
#define CMD_Weigh              108
#define CMD_Properties         109

#define CMD_SelectAll          110
#define CMD_UnSelectAll        111
#define CMD_SelectSameExt      112
#define CMD_UnSelectSameExt    113
#define CMD_SelectGroup        114
#define CMD_UnSelectGroup      115

#define CMD_CreateNewArchive   114
#define CMD_AddToArchive       115
#define CMD_ExtractArchive     116
#define CMD_ExtractTo          117
#define CMD_TestArchive        118


/**
 * Author:
 * Piotr Mierzwiski
 **/
/** @short Klasa obsluguje menu podrczne panela plikw.
 Inicjowane jest tutaj menu podrczne standardowymi opcjami, po wybraniu opcji
 wysyany jest odpowiedni sygna dla niej.
*/
class PopupMenu : public QPopupMenu  {
   Q_OBJECT
public:
	/** Rodzaj menu.
	 */
	enum KindOfPopupMenu { FilesPanelMENU, UserFilesPanelMENU };

	/** Konstruktor klasy.\n
	 @param parent - wskanik na rodzica,
	 @param kindOfPopupMenu - rodzaj menu, patrz: @see KindOfPopupMenu,
	 @param name - nazwa dla obiektu.\n
	 Inicjowane s tu skadowe, inicjowane s opcje dla menu oraz czone sygnay
	 ze slotami.
	 */
	PopupMenu( QWidget *parent=0, KindOfPopupMenu kindOfPopupMenu=FilesPanelMENU, const char *name=0 );

	/** Destruktor.\n
	 Brak definicji.
	 */
	~PopupMenu();

	/** Pokazuje dialog menu na podanej pozycji.\n
	 @param globalPos - pozycja dla menu.
	 */
	bool showMenu( const QPoint & globalPos );

	/** Funkcja zawiera obsug klawiatury dla menu.\n
	 @param e - zdarzenie klawiatury.
	 Zawiera podstawow obsg klawiatury, tj. poruszanie kursorem
	 (gra/d), litera skrtu. Nie jest obsugiwane podmenu.
	 */
	void keyPressed( QKeyEvent * e );

private:
	QString mCurrentItemName;
	int mHighlightedItemNum;
	bool mMoveFile;

	/** Inicjowane s tutaj opcje dla standardowego menu panela plikw.
	 */
	void initFilesPanelMenu();

	/** Inicjowane s tutaj opcje dla menu panela plikw - zdefiniowane przez
	 uytkownika.
	 */
	void initUserMenu();

	/** Uruchamia akcje okrelon w menu.\n
	 @param cmd - kod polecenia z menu.\n
	 Wysyane s tutaj odpowiadajce opcj sygnay.
	 */
	void execCmd( int cmd );

private slots:
	/** Slot wywoywany po podwietleniu opcji w menu.\n
	 @param itemId - kod polecenia z menu.\n
	 Inicjowana jest tutaj tylko skadowa zawierajca kod opcji.
	 */
	void slotItemHighlighted( int itemId );

//protected:
//	void keyPressed( QKeyEvent * );

signals:
	/** Sygna wysyany po wybraniu opcji 'Open'.\n
	 @param item - wskanik na element do otwarcia (0 oznacza biecy).\n
	 Uruchamia procedur otwarcia pliku lub katalogu.
	 */
	void signalOpen( QListViewItem * item );

	/** Sygna wysyany po wybraniu opcji 'OpenInPanelBeside'.\n
	 @param dirForce - TRUE oznacza otwarcie katalogu (jeli kursor nie stoi
	 na elemencie bdcym katalogiem, wtedy otwierany jest biecy), FALSE
	 powoduje otwarcie pliku w panelu obok.\n
	 Uruchamia procedur otwarcia pliku lub katalogu w panelu obok.
	 */
	void signalOpenInPanelBeside( bool dirForce );

	/** Sygna wysyany po wybraniu opcji 'Rename'.
	 @param column - numer kolumny w ktrej naley zmieni warto (tutaj
	 zawsze podawane jest 0, oznaczajce kolumn z nazw).\n
	 Uruchamia procedur zmiany nazwy pliku.
	 */
	void signalRename( int column );

	/** Sygna wysyany po wybraniu opcji 'Delete'.\n
	 Uruchamia procedur usunicia biecego elementu.
	 */
	void signalDelete();

	/** Sygna wysyany po wybraniu opcji 'MakeLink'.\n
	 Uruchamia procedur utworzenia linku dla biecego elementu.
	 */
	void signalMakeLink();

	/** Sygna wysyany po wybraniu opcji 'Copy'.\n
	 @param shiftPressed - TRUE oznacza, e wcinity jest klawisz Shift.
	 @param move - TRUE wymusza przenoszenie, w przeciwnym razie jest to
	 kopiowanie.\n
	 Uruchamia procedur kopiowania lub przenoszenia zaznaczonych lub
	 biecego elementu.
	 */
	void signalCopy( bool shiftPressed, bool move );

	/** Sygna wysyany po wybraniu opcji 'SelectAll' lub 'UnSelectAll'.\n
	 @param select - TRUE oznacza, e wykonywane bdzie zaznaczenie, w przeciwnym
	 razie elementy zostan odznaczone.\n
	 Uruchamia procedur zaznaczenia lub odznaczenia wszystkich elementw.
	 */
	void signalSelectAll( bool select );

	/** Sygna wysyany po wybraniu opcji 'SelectSameExt'.\n
	 @param select - TRUE oznacza, e wykonywane bdzie zaznaczenie, w przeciwnym
	 razie elementy zostan odznaczone.\n
	 Uruchamia procedur zaznaczenia lub odznaczenia elementw z tym samym
	 rozszerzeniem, jakie ma biecy plik.
	 */
	void signalSelectSameExt( bool select );

	/** Sygna wysyany po wybraniu opcji 'SelectGroup' lub 'UnSelectGroup'.\n
	 @param select - TRUE oznacza, e wykonywane bdzie zaznaczenie, w przeciwnym
	 razie elementy zostan odznaczone.\n
	 Uruchamia procedur powodujc pokazanie dialogu zaznacze grupowych
	 (podawany jest tu wzorzec, ktry bdzie dopasowywany do plikw).
	 */
	void signalSelectGroup( bool select );

	/** Sygna wysyany po wybraniu opcji 'Properties'.\n
	 Uruchamia procedur powodujc pokazanie dialogu waciwoci dla zaznaczonych
	 lub biecego pliku.
	 */
	void signalShowProperties();

//	void signalCreateNewArchive();
//	void signalAddToArchive();
//	void signalExtractArchive();
//	void signalExtractTo();
//	void signalTestArchive();

};

#endif
