/***************************************************************************
                          tar.h  -  description
                             -------------------
    begin                : tue dec 23 2003
    copyright            : (C) 2003 by Piotr Mierzwiski
    email                : peterm@go2.pl

    copyright            : See COPYING file that comes with this project
 ***************************************************************************/

#ifndef _TAR_H_
#define _TAR_H_

#include "enums.h"
#include "virtualarch.h"

/**
 * Author:
 * Piotr Mierzwiski
 **/
/** @short Klasa obsugujca archiwa tar, tar.bz2 oraz tar.gz.
 Przygotowywane s tutaj polecenia wykonywane na archiwach (np. listowania),
 ktre s pniej uruchamiane na rzecz archiwum. Polecenia wykonywane s
 asynchronicznie (po uruchomieniu nie blokuj interfejsu uytkownika). Klasa
 zajmuje si take wstpn obrbk danych wyjciowych podstawowych w wyniku
 uruchomienia podanego polecenia, np. listowania, wypakowywania, itp.
*/
class Tar : public VirtualArch
{
	Q_OBJECT
public:
	/** Konstruktor klasy.\n
	 @param parent - wskanik na rodzica,
	 @param kindOfArchive - rodzaj archiwum, patrz typ @see KindOfArchive.
	 @param compressionLevel - poziom kompresji,
	 @param name - nazwa dla obiektu.\n
	 Inicjowane s tutaj niezbdne skadowe klasy.
	 */
	Tar( QWidget *parent, KindOfArchive kindOfArchive, int compressionLevel=0, const char *name=0 );

	/** Destruktor.\n
	 Brak definicji.
	 */
	~Tar() {}

	/** Metoda powoduje asynchroniczne uruchamienie procesu.\n
	 @param archOperation - operacja do wykonania na archiwum
	 @param filesList - lista plikw, na ktrej naley wykona podan oparacj
	 */
	void runProcess( ArchOperation archOperation, const QStringList & filesList );

	/** Metoda wykonuje zmian katalogu w archiwum.\n
	 @param dirName - nazwa nowego katalogu w archiwum.\n
	 Przy czym zmiana, nie powoduje listowania.
	 */
	void cd( const QString & dirName );

	/** Metoda powoduje listowanie podanego archiwum.\n
	 @param dirName nazwa archiwum do wylistowania, przy czym moe by tu rwnie
	 podana nazwa katalogu z wewntrza archiwum.
	 */
	void list( const QString & dirName );

	/** Lista plikw z archiwum dla podgldu pliku przy uyciu klawisza F3.\n
	 @param archiveName - nazwa archiwum.\n
	 Uwaga. Brak definicji.
	 */
	void quickList( const QString & archiveName ) {}

	/** Metoda uruchamia doczanie podanych plikw do archiwum.\n
	 @param filesList - lista nazw plikw ktre naley doczy do archiwum.
	 */
	void append( const QStringList & filesList );

	/** Metoda uruchamia usunicie podanych plikw z archiwum.\n
	 @param filesList - lista nazw plikw do usunicia z archiwum.
	 */
	void remove( const QStringList & filesList );

	/** Metoda uruchamia wypakowywanie podanych plikw z archiwum.\n
	 @param filesList - lista plikw, ktre naley wypakowa z archiwum,
	 @param all - rwne TRUE, oznacza e naley wypakowa cae archiwum.
	 */
	void extract( const QStringList & filesList, bool all=FALSE );

	/** Zwraca kod bdu dla ostatniej operacji na archiuwm.
	 @return kod bdu.
	 */
	Error errorCode() const { return mErrorCode; }

	/** Zwraca nazw aktualnie przetwarzanego pliku z absolutn ciek.\n
	 @return nazwa aktualnie przetwarzanego pliku.
	 */
	QString nameOfProcessedFile() const { return mNameOfProcessedFile; }

	/** Zwraca biecy stan operacji na archiwum.\n
	 @return stan operacji na archiwum (warto typu @see VirtualArch::ArchState).
	 */
	int state() const { return mState; }

	/** Pobierane s tu wyniki operacji waenia, tj. waga caoci, ilo plikw
	 i katalogw \n
	 @param weight - zmienna, w ktrej zapisana bdzie waga,
	 @param files - zmienna do zapisania iloci plikw,
	 @param dirs - zmienna do zapisania iloci katalogw.
	 */
	void getWeighingStat( long long & weigh, uint & files, uint & dirs );

private:
	Error mErrorCode;
	QString mNameOfProcessedFile;
	QStringList mArgumentsList, mListFilesToProcessed, mFilesListBuffer;
	ArchOperation mArchOperation;

	KindOfArchive mKindOfArchive;
	int mCompressionLevel, mDirHierachyLevel;
	ArchState mState;

	QString mBuffer, mErrBuffer;

protected:
	/** Zwraca identyfikator biecej operacji na archiwum.\n
	 @return identyfikator biecej operacji.
	 */
	ArchOperation archiveOperation() const { return mArchOperation; }

private slots:
	/** Slot wywoywany w momencie przerywania dzialania procesu.
	 */
	void slotBreakProcess();

	/** Slot wywoywany w chwili zakoczenia dziaania uruchomionego procesu.\n
	 Zawarta jest tutaj wstpna obsuga bdw po nie udanym zakoczeniu procesu.
	 Dla polecenia 'OPEN' wysywane s sygnay: @em commandFinished() oraz
	 @em done(), po czym uruchamiane polecenie listowania.
	 */
	void slotProcessExited();

	/** Slot wywoywany w chwili przesyania danych przez proces na standardowe
	 wyjcie.
	 */
	void slotReadFromStdout();

	/** Slot wywoywany, gdy proces przesya dane na standardowe wyjcie bdu.
	 */
	void slotReadFromStderr();

	/** W slocie ustawiane s argumenty dla procesu, po czym jest on uruchamiany.\n
	 W przypadku niepowodzenia wysyany jest sygna @em commandFinished(), a kod
	 bdu przybiera warto 'CannotRunProc'.
	 */
	void slotRunProcess();

	/** Nastpuje tutaj przetworzenie bufora wyjciowego po operacji listowania.\n
	Poszczeglne pola z informacj o pliku s ustawiane w kolejnoci wymaganej
	przez metode wydobywajc pliki z okrelonego poziomu w hierarchi katalogw
	w archiwum przez metod @em VirtualArch::listFiles(). Kada linia dodawana
	jest do listy cigw (skadowa klasy), przekazywanej do wczeniej wymienionej
	metody.\n Slot jest wywolywany w sposb nie blokujcy interfejsu uytkownika.
	\n \n Patrz te: @see VirtualArch::listFiles().
	 */
	void slotOpen();

};

#endif
